import {
  select,
  csv, 
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  extent, 
  axisLeft,
  axisBottom,
  line,
  curveBasis,
  nest,
  schemeCategory10,
  descending
} from 'd3';
import { getData } from './data';
import { colorLegend } from './legend';

// Get the SVG from the DOM
const svg = select('svg');

// Extract Width and Height from SVG and parse it to float
const height = +svg.attr('height');
const width  = +svg.attr('width');
const margin = {top: 65, right: 175, bottom: 90, left: 115};
const innerWidth = width-margin.left-margin.right;
const innerHeight = height-margin.top-margin.bottom;

const render = data => {
  
   const xValue = d => d.timestamp;
  const xAxisLabel = 'Time';
  
  const yValue = d => d.temperature;
  const yAxisLabel = 'Temperature';
  
  const colorValue = d => d.city;

  const titleLabel = `${xAxisLabel} versus ${yAxisLabel}`;
  
  const lastTemp = d => yValue(d.values[d.values.length-1]);
  
  const nested = nest()
  	.key(colorValue)
  	.entries(data)
  	.sort((a,b ) => descending(lastTemp(a),lastTemp(b)));


  // Set Scales of X and Y Axis
  const xScale = scaleTime()
  	.domain(extent(data, xValue))
  	.range([0,innerWidth])
  	.nice();
  const yScale = scaleLinear()
  	.domain(extent(data, yValue))
  	.range([innerHeight,0])
  	.nice();
  
  const colorScale = scaleOrdinal(schemeCategory10)
  	.domain(nested.map(d=>d.key));
  
  // Format X Axis
  const xAxis = axisBottom(xScale)
  	.tickSize(-innerHeight)
  	.tickPadding(20);
  
  // Format Y Axis
  const yAxis = axisLeft(yScale)
  	.tickSize(-innerWidth)
  	.tickPadding(10);

  // Add Main Bar Group
  const lineGroup = svg
  	.append('g')
  		.attr('transform', `translate(${margin.left}, ${margin.top})`);

  // Add Y Axis Group
  const yAxisGroup = lineGroup
    .append('g')
    	.call(yAxis);
  yAxisGroup
			.select('.domain')
    		.remove();
  yAxisGroup
    .append('text')
  		.attr('class','axisLabel')
  		.attr('x',-(innerHeight/2))
  		.attr('y',-60)
  		.attr('transform', 'rotate(-90)')
  		.attr('fill', 'black')
    	.attr('text-anchor', 'middle')
  		.text(yAxisLabel);
  
  // Add X Axis Group
  const xAxisGroup = lineGroup
    .append('g')
    	.call(xAxis)
  		.attr('transform', `translate(0, ${innerHeight})`);
 	xAxisGroup
   .select('.domain')
  		.remove();
 	xAxisGroup
    .append('text')
  		.attr('class','axisLabel')
  		.attr('x',innerWidth/2)
  		.attr('y',margin.bottom-10)
  		.attr('fill', 'black')
  		.text(xAxisLabel);

  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  

  
  // Add The Path
  lineGroup.selectAll('.line-path').data(nested)
  	.enter().append('path')
  		.attr('class','line-path')
  		.attr('d',d => lineGenerator(d.values))
  		.attr('stroke', d => colorScale(d.key));
  

  // Add The Title
  lineGroup.append('text')
		.attr('class','titleLabel')
  	.attr('x',innerWidth/2)
  	.attr('y', -20)
  	.attr('text-anchor', 'middle')
  	.text(titleLabel);
  
  svg.append('g')
	.attr('transform',`translate(800,150)`)
	.call(colorLegend, {
		colorScale,
    itemRadius: 10,
  	itemSpacing: 25,
    itemTextPos: 15
});

};

getData()
  .then(data => {
//  	console.log(data);
  	render(data);
});

